@@ -6,6 +6,7 @@ const StorageFolder = require('../storage-folder')
 const Config = require('../config')
 const ConfigFile = require('../config-file')
 const FileRecoveryService = require('./file-recovery-service')
+const StartupTime = require('../startup-time')
 const ipcHelpers = require('../ipc-helpers')
 const {BrowserWindow, Menu, app, clipboard, dialog, ipcMain, shell, screen} = require('electron')
 const {CompositeDisposable, Disposable} = require('event-kit')
@@ -92,6 +93,9 @@ const encryptOptions = (options, secret) => {
 
   // Even if the following IV is not cryptographically secure, there's a really good chance
   // it's going to be unique between executions which is the requirement for GCM.
+  // We're not using `crypto.randomBytes()` because in electron v2, that API is really slow
+  // on Windows machines, which affects the startup time of Atom.
+  // TodoElectronIssue: Once we upgrade to electron v3 we can use `crypto.randomBytes()`
   const initVectorHash = crypto.createHash('sha1')
   initVectorHash.update(Date.now() + '')
   initVectorHash.update(Math.random() + '')
@@ -132,6 +136,8 @@ module.exports =
 class AtomApplication extends EventEmitter {
   // Public: The entry point into the Atom application.
   static open (options) {
+    StartupTime.addMarker('main-process:atom-application:open')
+
     const socketSecret = getExistingSocketSecret(options.version)
     const socketPath = getSocketPath(socketSecret)
     const createApplication = options.createApplication || (async () => {
@@ -169,6 +175,8 @@ class AtomApplication extends EventEmitter {
   }
 
   constructor (options) {
+    StartupTime.addMarker('main-process:atom-application:constructor:start')
+
     super()
     this.quitting = false
     this.quittingForUpdate = false
@@ -211,6 +219,8 @@ class AtomApplication extends EventEmitter {
 
     this.disposable = new CompositeDisposable()
     this.handleEvents()
+
+    StartupTime.addMarker('main-process:atom-application:constructor:end')
   }
 
   // This stuff was previously done in the constructor, but we want to be able to construct this object
@@ -218,6 +228,8 @@ class AtomApplication extends EventEmitter {
   // of these various sub-objects into the constructor, but you'll need to remove the side-effects they
   // perform during their construction, adding an initialize method that you call here.
   async initialize (options) {
+    StartupTime.addMarker('main-process:atom-application:initialize:start')
+
     global.atomApplication = this
 
     // DEPRECATED: This can be removed at some point (added in 1.13)
@@ -232,6 +244,9 @@ class AtomApplication extends EventEmitter {
 
     // Don't await for the following method to avoid delaying the opening of a new window.
     // (we await it just after opening it).
+    // We need to do this because `listenForArgumentsFromNewProcess()` calls `crypto.randomBytes`,
+    // which is really slow on Windows machines.
+    // (TodoElectronIssue: This got fixed in electron v3: https://github.com/electron/electron/issues/2073).
     const socketServerPromise = this.listenForArgumentsFromNewProcess(options)
 
     this.setupDockMenu()
@@ -240,6 +255,8 @@ class AtomApplication extends EventEmitter {
     this.autoUpdateManager.initialize()
     await socketServerPromise
 
+    StartupTime.addMarker('main-process:atom-application:initialize:end')
+
     return result
   }
 
@@ -254,10 +271,17 @@ class AtomApplication extends EventEmitter {
 
   async launch (options) {
     if (!this.configFilePromise) {
-      this.configFilePromise = this.configFile.watch()
-      this.disposable.add(await this.configFilePromise)
-      this.config.onDidChange('core.titleBar', () => this.promptForRestart())
-      this.config.onDidChange('core.colorProfile', () => this.promptForRestart())
+      this.configFilePromise = this.configFile.watch().then(disposable => {
+        this.disposable.add(disposable)
+        this.config.onDidChange('core.titleBar', () => this.promptForRestart())
+        this.config.onDidChange('core.colorProfile', () => this.promptForRestart())
+      })
+
+      // TodoElectronIssue: In electron v2 awaiting the watcher causes some delay
+      // in Windows machines, which affects directly the startup time.
+      if (process.platform !== 'win32') {
+        await this.configFilePromise
+      }
     }
 
     let optionsForWindowsToOpen = []
@@ -659,19 +683,23 @@ class AtomApplication extends EventEmitter {
 
     // A request from the associated render process to open a set of paths using the standard window location logic.
     // Used for application:reopen-project.
-    this.disposable.add(ipcHelpers.on(ipcMain, 'open', (_event, options) => {
+    this.disposable.add(ipcHelpers.on(ipcMain, 'open', (event, options) => {
       if (options) {
         if (typeof options.pathsToOpen === 'string') {
           options.pathsToOpen = [options.pathsToOpen]
         }
 
+        if (options.here) {
+          options.window = this.atomWindowForEvent(event)
+        }
+
         if (options.pathsToOpen && options.pathsToOpen.length > 0) {
           this.openPaths(options)
         } else {
           this.addWindow(this.createWindow(options))
         }
       } else {
-        this.promptForPathToOpen('all', {window})
+        this.promptForPathToOpen('all', {})
       }
     }))
 
@@ -888,6 +916,7 @@ class AtomApplication extends EventEmitter {
   // Returns the {AtomWindow} for the given locations.
   windowForLocations (locationsToOpen, devMode, safeMode) {
     return this.getLastFocusedWindow(window =>
+      !window.isSpec &&
       window.devMode === devMode &&
       window.safeMode === safeMode &&
       window.containsLocations(locationsToOpen)
@@ -1043,13 +1072,13 @@ class AtomApplication extends EventEmitter {
       // focused window that matches the requested dev and safe modes.
       if (!existingWindow && addToLastWindow) {
         existingWindow = this.getLastFocusedWindow(win => {
-          return win.devMode === devMode && win.safeMode === safeMode
+          return !win.isSpec && win.devMode === devMode && win.safeMode === safeMode
         })
       }
 
       // Fall back to the last focused window that has no project roots.
       if (!existingWindow) {
-        existingWindow = this.getLastFocusedWindow(win => !win.hasProjectPaths())
+        existingWindow = this.getLastFocusedWindow(win => !win.isSpec && !win.hasProjectPaths())
       }
 
       // One last case: if *no* paths are directories, add to the last focused window.
@@ -1057,7 +1086,7 @@ class AtomApplication extends EventEmitter {
         const noDirectories = locationsToOpen.every(location => !location.isDirectory)
         if (noDirectories) {
           existingWindow = this.getLastFocusedWindow(win => {
-            return win.devMode === devMode && win.safeMode === safeMode
+            return !win.isSpec && win.devMode === devMode && win.safeMode === safeMode
           })
         }
       }
@@ -1066,6 +1095,7 @@ class AtomApplication extends EventEmitter {
     let openedWindow
     if (existingWindow) {
       openedWindow = existingWindow
+      StartupTime.addMarker('main-process:atom-application:open-in-existing')
       openedWindow.openLocations(locationsToOpen)
       if (openedWindow.isMinimized()) {
         openedWindow.restore()
@@ -1090,6 +1120,7 @@ class AtomApplication extends EventEmitter {
       if (!resourcePath) resourcePath = this.resourcePath
       if (!windowDimensions) windowDimensions = this.getDimensionsForNewWindow()
 
+      StartupTime.addMarker('main-process:atom-application:create-window')
       openedWindow = this.createWindow({
         locationsToOpen,
         windowInitializationScript,
